//SPDX-FileCopyrightText: 2024 Yury Gubich <blue@macaw.me>
//SPDX-License-Identifier: GPL-3.0-or-later

#include "deletetransaction.h"

#include "server/server.h"
#include "server/session.h"
#include "database/exceptions.h"

Handler::DeleteTransaction::DeleteTransaction (const std::shared_ptr<Server>& server):
    Handler("deleteTransaction", Request::Method::post),
    server(server)
{}

void Handler::DeleteTransaction::handle (Request& request) {
    std::string access = request.getAuthorizationToken();
    if (access.empty())
        return error(request, Response::Status::unauthorized);

    if (access.size() != 32)
        return error(request, Response::Status::badRequest);

    std::shared_ptr<Server> srv = server.lock();
    if (!srv)
        return error(request, Response::Status::internalError);

    std::map form = request.getForm();
    std::map<std::string, std::string>::const_iterator itr = form.find("id");
    if (itr == form.end())
        return error(request, Response::Status::badRequest);

    unsigned int txnId;
    try {
        txnId = std::stoul(itr->second);
    } catch (const std::exception& e) {
        return error(request, Response::Status::badRequest);
    }

    try {
        Session& session = srv->getSession(access);
        bool success = srv->getDatabase()->deleteTransaction(txnId, session.owner);
        if (!success)
            return error(request, Response::Status::forbidden);

        Response& res = request.createResponse(Response::Status::ok);
        res.send();

        session.transactionRemoved(txnId);

    } catch (const DB::NoSession& e) {
        return error(request, Response::Status::unauthorized);
    } catch (const std::exception& e) {
        std::cerr << "Exception on " << path << ":\n\t" << e.what() << std::endl;
        return error(request, Response::Status::internalError);
    }  catch (...) {
        std::cerr << "Unknown exception on " << path << std::endl;
        return error(request, Response::Status::internalError);
    }
}
